<?php

	/**
	 * Klasa zarządzająca obiektami graficznymi z wykorzystaniem biblioteki GD
	 * @author Wojciech Król <wk20981@gmail.com>
	 * @copyright Copyright (c) 2014, Wojciech Król, Wszelkie prawa zastrzeżone
	 * @link http://wojciechkrol.eu Wojciech Król - Portfolio
	 * @version 1.0
	 */
	
	class GDImage {
		
		protected $defaultImageProperties = array(
			"width" => 800, // Szerokość obiektu
			"height" => 600, // Wysokość obiektu
			"backgroundColor" => "#000", // Kolor tła
			"backgroundImage" => null, // Obazek jako tło
			"setBackgroundImageSize" => false, // Ustawienie wymiarów obrazu jako wymiary obiektu - podane wyżej wymiary są ignorowane
			"padding" => array(0,0,0,0) // Odstęp obszaru obiektu od krawędzi (przydatne przy tworzeniu ramki)
		);
		protected $defaultTextProperties = array(
			"x" => 0, // Pozycja 'x' tekstu
			"y" => 0, // Pozycja 'y' tekstu
			"maxWidth" => "auto", // Maksymalna szerokość tekstu w px
			"lineSpacing" => 3, // Odstęp między wierszami
			"padding" => array(0,0,0,0), // Odstęp tesktu od krawędzi
			"angle" => 0, // Kąt wyświetalania
			"color" => "#FFF", // Kolor tekstu
			"fontSize" => 14, // Wielkość czcionki w px
			"fontFile" => "files/Lobster.ttf", // Ścieżka do używanego fontu
			"align" => "left", // Wyrównanie tekstu w poziomie
			"verticalAlign" => "top", // Wyrównanie tekstu w pionie
			"strokeSize" => 0, // Wielkość obrysowania
			"strokeColor" => "#000" // Kolor obrysowania
		);
		
		protected $imageProperties = null;
		public $resource = null;
		
		public function __construct($properties = array()) {
			
			// Properties
			$p = array_merge($this->defaultImageProperties, $properties);
			$bgColor = $this->_hex2rgb($p['backgroundColor']);
			
			// Tworzenie obiektu z jednolitym tłem
			if(empty($p['backgroundImage'])) {
				$this->resource = imagecreatetruecolor($p['width'], $p['height']);
				$bg = imagecolorallocate($this->resource, $bgColor[0], $bgColor[1], $bgColor[2]);
				imagefill($this->resource, 0, 0, $bg);
			}
			// Tworzenie obiektu z obrazka
			else {
				// Tworzenie obiektu o wymiarach obrazka
				if($p['setBackgroundImageSize']) {
					$this->resource = imagecreatefromstring(file_get_contents($p['backgroundImage']));
				}
				// Tworzenie obiektu o ustalonych wymiarach i najlepsze dopasowanie obrazka
				else {
					$this->resource = imagecreatetruecolor($p['width'], $p['height']);
					$tmp_image = imagecreatefromstring(file_get_contents($p['backgroundImage']));
					$image_width = imagesx($tmp_image);
					$image_height = imagesy($tmp_image);
					
					$ratio = $image_width/$image_height;
					$objectRatio = $p['width']/$p['height'];
					
					if($ratio > 1 && $objectRatio < $ratio) {
						$new_width = round($p['height']*$ratio);
						$new_height = $p['height'];
						
						$src_x = round(($image_width - $image_height*$objectRatio) / 2);
						$src_y = 0;
					}
					else {
						$new_width = $p['width'];
						$new_height = round($p['width']/$ratio);
						$h = ($image_height>$p['height'])?$new_height:$image_height;
						$src_x = 0;
						$src_y = round(($image_height - $image_width/$objectRatio) / 2);
					}
					
					imagecopyresampled(
						$this->resource,
						$tmp_image,
						0,
						0,
						$src_x,
						$src_y,
						$new_width,
						$new_height,
						$image_width,
						$image_height
					);
				}
			}
			
			$this->imageProperties = array_merge($p, array(
				'width' => imagesx($this->resource),
				'height' => imagesy($this->resource)
			));
			
			imagealphablending($this->resource, true);
			imagesavealpha( $this->resource, true );
			
			if(array_sum($this->imageProperties['padding']) > 0)
			{
				$tmp = imagecreatetruecolor(
					$this->imageProperties['width'] + $this->imageProperties['padding'][1] + $this->imageProperties['padding'][3],
					$this->imageProperties['height'] + $this->imageProperties['padding'][0] + $this->imageProperties['padding'][2]
				);
				$r = $this->_hex2rgb($this->imageProperties['backgroundColor']);
				$color = imagecolorallocate($tmp, $r[0], $r[1], $r[2]);
				imagefill($tmp, 0, 0, $color);
				imagecopy($tmp, $this->resource, $this->imageProperties['padding'][3], $this->imageProperties['padding'][0], 0, 0, $this->imageProperties['width'], $this->imageProperties['height']);
				$this->resource = $tmp;
				
				$this->imageProperties = array_merge($p, array(
					'width' => imagesx($this->resource),
					'height' => imagesy($this->resource)
				));
			}
			
		}
		
		public function resizeTo($width = null, $height = null)
		{
			if(!empty($width) && !empty($height))
			{
				$tmp = imagecreatetruecolor($width, $height);
				imagecopyresampled($tmp, $this->resource, 0, 0, 0, 0, $width, $height, $this->getProperty('width'), $this->getProperty('height'));
				$this->resource = $tmp;
			}
			else {
				if(!empty($width))
				{
					$ratio = $this->getProperty('width')/$this->getProperty('height');
					$height = $width/$ratio;
					
					$tmp = imagecreatetruecolor($width, $height);
					imagecopyresampled($tmp, $this->resource, 0, 0, 0, 0, $width, $height, $this->getProperty('width'), $this->getProperty('height'));
					$this->resource = $tmp;
				}
				else if(!empty($height))
				{
					$ratio = $this->getProperty('width')/$this->getProperty('height');
					$width = $height*$ratio;
					
					$tmp = imagecreatetruecolor($width, $height);
					imagecopyresampled($tmp, $this->resource, 0, 0, 0, 0, $width, $height, $this->getProperty('width'), $this->getProperty('height'));
					$this->resource = $tmp;
				}
			}
			
			$this->imageProperties = array_merge($this->imageProperties, array(
				'width' => imagesx($this->resource),
				'height' => imagesy($this->resource)
			));
		}
		
		/**
		 * Metoda pobiera wartości z tablicy właściwości obiektu
		 * @param string $name nazwa właściwości.
		 * @param mixed $arrayKey klucz dla tablicy zagnieżdżonej we właściwości.
		 * @return mixed Zwraca wartość z tablicy.
		 */
		
		public function getProperty($name, $arrayKey = null) {
			if(isset($this->imageProperties[$name]))
			{
				if($arrayKey === null)
					return $this->imageProperties[$name];
				else
					return $this->imageProperties[$name][$arrayKey];
			}
			
			return false;
		}
		
		/** 
		 * Wyświetlenie tekstu na obiekcie
		 * @param string $text wyświetlany tekst
		 * @param array $properties style
		 */
		
		public function text($text, $properties = array(), $expand = false, $padding = array(0,0))
		{
			$p = array_merge($this->defaultTextProperties, $properties);
			if($p['maxWidth'] == "auto") {
				$p['maxWidth'] = $this->getProperty('width');
			}
			if(!file_exists($p['fontFile'])) return;
			
			// Wrapping
			$height = 0;
			$text = str_replace("\n", " {EOL} ", $text);
			$word_array = explode(" ", $text);
			$line = array();
			$current_line = 0;
			$line_info = array();
			
			foreach($word_array as $word) {
				$line[ $current_line ][] = $word;
				$info = imagettfbbox($p['fontSize'], $p['angle'], $p['fontFile'], implode(" ", $line[ $current_line ]) );
				if( ($info[4] + 2*$p['strokeSize'] + $p['padding'][1] + $p['padding'][2]) > $p['maxWidth'] || $word == "{EOL}") {
					array_pop($line[ $current_line ]);
						
					$line_info[ $current_line ] = imagettfbbox($p['fontSize'], $p['angle'], $p['fontFile'], implode(" ", $line[ $current_line ]) );
					$current_line++;
					
					if($word != "{EOL}")
						$line[ $current_line ][] = $word;
				}
				if(end($word_array) == $word) {
					$line_info[ $current_line ] = imagettfbbox($p['fontSize'], $p['angle'], $p['fontFile'], implode(" ", $line[ $current_line ]) );
				}
			}
			
			// looking for max line height
			$max_line_height = 0;
			foreach($line_info as $li) {
				if(abs($li[7]-$li[1]) > $max_line_height)
					$max_line_height = abs($li[7]-$li[1]);
			}
			foreach($line_info as $k => $li)
			{
				$height += abs($li[7]-$li[1])*1.1 + $p['lineSpacing'] + 2*$p['strokeSize'];
			}
			
			$rgbColor = $this->_hex2rgb($p['color']);
			$rgbStrokeColor = $this->_hex2rgb($p['strokeColor']);
			$color = imagecolorallocate($this->resource, $rgbColor[0], $rgbColor[1], $rgbColor[2]);
			$strokeColor = imagecolorallocate($this->resource, $rgbStrokeColor[0], $rgbStrokeColor[1], $rgbStrokeColor[2]);
			
			if($expand)
			{
				if(($p['y']+$height) > $this->getProperty('height'))
				{
					$tmp = imagecreatetruecolor($this->getProperty('width'), $p['y']+$height+$padding[0]);
					$bgColor = $this->_hex2rgb($this->getProperty("backgroundColor"));
					$bg = imagecolorallocate($tmp, $bgColor[0], $bgColor[1], $bgColor[2]);
					imagefill($tmp, 0, 0, $bg);
					imagecopy($tmp, $this->resource, 0, 0, 0, 0, $this->imageProperties['width'], $this->imageProperties['height']);
					$this->resource = $tmp;
				}
				$this->imageProperties = array_merge($this->imageProperties, array(
					'width' => imagesx($this->resource),
					'height' => imagesy($this->resource)
				));
			}
			foreach($line as $k => $l) {
				switch($p['align']) {
					case 'left': // Wyrównuje tekst do lewej krawędzi
						$cx = $p['x'] + 2*$p['strokeSize'];
						break;
					case 'center': // Wyśrodkowuje tekst w poziomie
						$cx = round($this->getProperty('width')/2 - ($line_info[$k][2] + $line_info[$k][0])/2);
						break;
					case 'right': // Wyrównuje tekst do prawej krawędzi
						$cx = $this->getProperty('width') - ($line_info[$k][2] - $line_info[$k][0])*1.02 - 2*$p['strokeSize'];
						break;
					default: // domyślnie do lewej krawędzi
						$cx = $p['x'] + 2*$p['strokeSize'];
				}
				switch($p['verticalAlign']) {
					case 'top': // Przykleja górną część obszaru tekstu tak, że tekst rozwijany jest w dół (parametr "y")
						$cy = $p['y'];
						break;
					case 'middle': // Ustawia środek obszaru tekstu na pozycji wartości parametru "y"
						$cy = $p['y'] - count($line)*($max_line_height + (2*$p['strokeSize'] + $p['lineSpacing']))/2;
						break;
					case 'bottom': // Przykleja dolną część obszaru tekstu tak, że jest rozwijany do góry
						$cy = $p['y'] - count($line)*($max_line_height + (2*$p['strokeSize'] + $p['lineSpacing']));
						break;
					default:
						$cy = $p['y'];
				}
				
				$cy = $cy + ($k+1)*($max_line_height) + $k*(2*$p['strokeSize'] + $p['lineSpacing']);
				
				$display_text = implode(" ", $l);
				
				if($p['strokeSize'] > 0) {
					$this->_strokeText($this->resource, $p['fontSize'], $p['angle'], $cx, $cy, $color, $strokeColor, $p['fontFile'], $display_text, $p['strokeSize']);
				}
				else
					imagettftext($this->resource, $p['fontSize'], $p['angle'], $cx, $cy, $color, $p['fontFile'], $display_text);
			}
			
			return $height;
		}
		
		/**
		 * Importowanie obiektu klasy GDImage i wklejanie do obiektu
		 * @param GDImage $resource obiekt klasy GDImage
		 * @param array $properties parametry importowania
		 */
		
		public function import(GDImage $resource, $properties = array())
		{
			$default = array(
				'x' => 0,
				'y' => 0,
				'opacity' => 100
			);
			
			$p = array_merge($default, $properties);
			imagecopymerge_alpha($this->resource, $resource->resource, $p['x'], $p['y'], 0, 0, $resource->getProperty('width'), $resource->getProperty('height'), $p['opacity']);
		}
		
		/**
		 * Eksport obiektu do obrazka i zapisanie go do pliku
		 * @param string $path ścieżka zapisu
		 * @param int $type format pliku: IMG_GIF | IMG_JPG | IMG_PNG | IMG_WBMP
		 * @param int $quality jakość obrazka w przypadku JPG
		 */
		
		public function export($path, $format = IMG_PNG, $quality = 90, $interlace = false)
		{
			imageinterlace($this->resource, (int)$interlace);
			switch($format) {
				case IMG_PNG:
					return imagepng($this->resource, $path, ceil(($quality/10) - 1));
				case IMG_JPG || IMG_JPEG:
					return imagejpeg($this->resource, $path, $quality);
				case IMG_GIF:
					return imagegif($this->resource, $path);
				case IMG_WBMP:
					return imagewbmp($this->resource, $path);
				default:
					return false;
			}
			
			return true;
		}
		
		/**
		 * Funkcja zapisuje obraz na dysku i wyświetla go na ekranie
		 * Pozwala to zaoszczędzić pamięć i dane do pobrania.
		 * @param string $path ścieżka zapisu
		 * @param int $type format pliku: IMG_GIF | IMG_JPG | IMG_PNG | IMG_WBMP
		 * @param int $quality jakość obrazka w przypadku JPG
		 */
		
		public function exportAndDisplay($path, $format = IMG_PNG, $quality = 90, $interlace = false)
		{
			$this->export($path, $format, $quality, $interlace);
			header("Content-type: ".$this->imageBitMimeType($format));
			readfile($path);
		}
		
		/**
		 * Wyświetlanie obrazka na ekranie
		 * @param int $format format wyświetlania IMG_PNG | IMG_JPG | IMG_GIF
		 */
		
		public function display($format = IMG_PNG, $quality = 90, $interlace = false)
		{
			header("Content-type: ".$this->imageBitMimeType($format)."; enctype=utf-8");
			imageinterlace($this->resource, (int)$interlace);
			switch($format) {
				case IMG_PNG:
					imagepng($this->resource, null, round($quality/10));
				case IMG_JPG || IMG_JPEG:
					imagejpeg($this->resource, null, $quality);
				case IMG_GIF:
					imagegif($this->resource);
				default:
					imagepng($this->resource);
			}
			imagedestroy($this->resource);
		}
		
		/**
		 * Metoda zwracająca typ mime bitu
		 * @param int $bit bit typu biblioteki GD
		 * @return string Mime type
		 */
		
		private function imageBitMimeType($bit)
		{
			switch($bit)
			{
				case IMG_GIF:
					return "image/gif";
				case IMG_JPG:
					return "image/jpeg";
				case IMG_PNG:
					return "image/png";
				default:
					return "image/png";
			}
		}
		
		/**
		 * Konwerter hex do rgb
		 * @param String $hexColor
		 * @return array tablica zawierająca trzy elementy: [0] - red, [1] - green, [2] - blue
		 */
		
		protected function _hex2rgb($hexColor)
		{
			$return = array(null,null,null);
			
			// Wydzielenie 
			$hexColor = str_replace("#","", $hexColor);
			
			if(strlen($hexColor) == 3) {
				$tmp = str_split($hexColor);
				$tmp[0] .= $tmp[0];
				$tmp[1] .= $tmp[1];
				$tmp[2] .= $tmp[2];
			}
			elseif(strlen($hexColor) == 6) {
				$tmp = str_split($hexColor, 2);
			}
			else {
				return false;
			}
			
			// Converting hexadecimal code to decimal code
			$return[0] = hexdec($tmp[0]);
			$return[1] = hexdec($tmp[1]);
			$return[2] = hexdec($tmp[2]);
			
			return $return;
		}
		
		/**
		 * Dodawania obrysowania do fontu
		 * @param resource $image obiektu typu resource pakietu GD
		 * @param int $size rozmiar czcionki
		 * @param int $angle kąt tekstu
		 * @param int $x położenie na osi x
		 * @param int $y położenie na osi y
		 * @param int $textcolor kolor tekstu
		 * @param int $strokecolor kolor obrysowania
		 * @param int $fontFile ścieżka do fontu
		 * @param int $text tekst
		 * @param int $px rozmiar obrysowania
		 * @return Returns an array with 8 elements representing four points making the bounding box of the text. The order of the points is lower left, lower right, upper right, upper left. The points are relative to the text regardless of the angle, so "upper left" means in the top left-hand corner when you see the text horizontally. Returns FALSE on error.
		 */
		
		protected function _strokeText(&$image, $size, $angle, $x, $y, &$textcolor, &$strokecolor, $fontfile, $text, $px) {
			for($a = -$px; $a <= $px; $a++)
				for($b =- $px; $b <= $px; $b++)
					if($b*$b + $a*$a <= $px*$px)
						$bg = imagettftext($image, $size, $angle, $x+$b, $y+$a, $strokecolor, $fontfile, $text);
		 
		   return imagettftext($image, $size, $angle, $x, $y, $textcolor, $fontfile, $text);
		}
	}
	
	// ALPHA FIX
	function imagecopymerge_alpha($dst_im, $src_im, $dst_x, $dst_y, $src_x, $src_y, $src_w, $src_h, $pct){ 
        $cut = imagecreatetruecolor($src_w, $src_h);
        imagecopy($cut, $dst_im, 0, 0, $dst_x, $dst_y, $src_w, $src_h); 
        imagecopy($cut, $src_im, 0, 0, $src_x, $src_y, $src_w, $src_h); 
        imagecopymerge($dst_im, $cut, $dst_x, $dst_y, 0, 0, $src_w, $src_h, $pct); 
    } 
?>